@@ -46,7 +46,9 @@ timetable's key.
 Objects that have timetables
 ----------------------------
 
-An object that has (or may have) timetables implements ITimetabled.
+An object that has (or may have) timetables implements
+IHaveTimetables.  The timetables themselves are stored in annotations
+and can be accessed through by adapting the owner to ITimetables.
 
 An object's composite timetable is derived by combining the object's
 timetable with composite timetables of other objects, acquired by
@@ -121,42 +123,33 @@ Timetable schemas
 A timetable schema is like a timetable that has no activities and no exeptions.
 You can create an empty timetable by calling the createTimetable method of a
 schema.  See ITimetableSchema, ITimetableSchemaDay.
-
-Terms
------
-
-A term defines a range in time (e.g. September 1 to December 31, 2004) and for
-every day within that range it defines whether that day is a schoolday or a
-holiday.
-
-
-$Id: __init__.py 4236 2005-07-20 14:30:51Z alga $
 """
 
-import datetime
-from sets import Set, ImmutableSet
-
+import rwproperty
 import zope.event
 from persistent import Persistent
-from persistent.list import PersistentList
 from persistent.dict import PersistentDict
-from zope.app.annotation.interfaces import IAttributeAnnotatable
+from zope.proxy import sameProxiedObjects
+from zope.component import queryAdapter
+from zope.component import adapter
+from zope.component import adapts, subscribers
+from zope.interface import implementer
 from zope.interface import implements
 from zope.interface import directlyProvides
-from zope.app import zapi
-from zope.app.traversing.api import getPath
-from zope.app.location.traversing import LocationPhysicallyLocatable
-from zope.component import provideAdapter, adapts
-from zope.app.container.btree import BTreeContainer
-from zope.app.container.contained import Contained
-from zope.app.location.interfaces import ILocation
+from zope.annotation.interfaces import IAnnotations
+from zope.location.interfaces import ILocation
+from zope.lifecycleevent.interfaces import IObjectAddedEvent
+from zope.lifecycleevent.interfaces import IObjectModifiedEvent
+from zope.traversing.api import getPath
+from zope.container.interfaces import INameChooser
+from zope.container.contained import NameChooser
+from zope.app.generations.utility import findObjectsProviding
 
-from schoolbell.app.cal import CalendarEvent
-from schoolbell.calendar.simple import ImmutableCalendar
-
-from schooltool.timetable.interfaces import ITimetableSchema
-from schooltool.timetable.interfaces import ITimetableSchemaDay
-from schooltool.timetable.interfaces import ITimetableSchemaWrite
+from schooltool.app.interfaces import ISchoolToolApplication
+from schooltool.app.interfaces import ISchoolToolCalendar
+from schooltool.calendar.simple import ImmutableCalendar
+from schooltool.common import IDateRange, DateRange
+from schooltool.timetable.interfaces import ITimetableCalendarEvent
 from schooltool.timetable.interfaces import ITimetable, ITimetableWrite
 from schooltool.timetable.interfaces import ITimetableDay, ITimetableDayWrite
 from schooltool.timetable.interfaces import ITimetableDict
@@ -164,204 +157,27 @@ from schooltool.timetable.interfaces import ITimetableActivity
 from schooltool.timetable.interfaces import ITimetableActivityAddedEvent
 from schooltool.timetable.interfaces import ITimetableActivityRemovedEvent
 from schooltool.timetable.interfaces import ITimetableReplacedEvent
-from schooltool.timetable.interfaces import ISchooldayPeriod
+from schooltool.timetable.interfaces import ISchooldaySlot
 from schooltool.timetable.interfaces import ISchooldayTemplate
 from schooltool.timetable.interfaces import ISchooldayTemplateWrite
-from schooltool.timetable.interfaces import ITimetabled
-from schooltool.timetable.interfaces import ITimetableSchemaContainer
-from schooltool.timetable.interfaces import ITermContainer
-from schooltool.timetable.interfaces import ITermWrite, ITerm
+from schooltool.timetable.interfaces import ITimetables, IHaveTimetables, IOwnTimetables
+from schooltool.timetable.interfaces import ICompositeTimetables
 from schooltool.timetable.interfaces import ITimetableSource
+from schooltool.timetable.interfaces import ITimetableSchema
 from schooltool.timetable.interfaces import Unchanged
-from schooltool.timetable.interfaces import IDateRange
-from schooltool import getSchoolToolApplication
-
-# Imports for ZODB compatibility
-from schooltool.timetable.model import BaseTimetableModel
-from schooltool.timetable.model import WeeklyTimetableModel
-from schooltool.timetable.model import SequentialDaysTimetableModel
-from schooltool.timetable.model import SequentialDayIdBasedTimetableModel
-from schooltool.timetable.model import TimetableCalendarEvent
-
-__metaclass__ = type
+from schooltool.term.interfaces import ITerm
+from schooltool.app.app import InitBase
+from schooltool.schoolyear.subscriber import ObjectEventAdapterSubscriber
 
 
+from schooltool.common import SchoolToolMessage as _
 
-#
-# Date ranges and terms
-#
-
-class DateRange:
-
-    implements(IDateRange)
-
-    def __init__(self, first, last):
-        self.first = first
-        self.last = last
-        if last < first:
-            # import timemachine
-            raise ValueError("Last date %r less than first date %r" %
-                             (last, first))
-
-    def __iter__(self):
-        date = self.first
-        while date <= self.last:
-            yield date
-            date += datetime.date.resolution
-
-    def __len__(self):
-        return (self.last - self.first).days + 1
-
-    def __contains__(self, date):
-        return self.first <= date <= self.last
-
-
-class Term(DateRange, Persistent):
-
-    implements(ITerm, ITermWrite, ILocation)
-
-    __name__ = None
-    __parent__ = None
-
-    def __init__(self, title, first, last):
-        DateRange.__init__(self, first, last)
-        self.title = title
-        self._schooldays = Set()
-
-    def _validate(self, date):
-        if not date in self:
-            raise ValueError("Date %r not in term [%r, %r]" %
-                             (date, self.first, self.last))
-
-    def isSchoolday(self, date):
-        self._validate(date)
-        if date in self._schooldays:
-            return True
-        return False
-
-    def add(self, date):
-        self._validate(date)
-        self._schooldays.add(date)
-        self._schooldays = self._schooldays  # persistence
-
-    def remove(self, date):
-        self._validate(date)
-        self._schooldays.remove(date)
-        self._schooldays = self._schooldays  # persistence
-
-    def addWeekdays(self, *weekdays):
-        for date in self:
-            if date.weekday() in weekdays:
-                self.add(date)
-
-    def removeWeekdays(self, *weekdays):
-        for date in self:
-            if date.weekday() in weekdays and self.isSchoolday(date):
-                self.remove(date)
-
-    def toggleWeekdays(self, *weekdays):
-        for date in self:
-            if date.weekday() in weekdays:
-                if self.isSchoolday(date):
-                    self.remove(date)
-                else:
-                    self.add(date)
-
-    def reset(self, first, last):
-        if last < first:
-            # import timemachine
-            raise ValueError("Last date %r less than first date %r" %
-                             (last, first))
-        self.first = first
-        self.last = last
-        self._schooldays.clear()
-
+##############################################################################
 
 #
 # Timetabling
 #
 
-class TimetableSchema(Persistent, Contained):
-
-    implements(ITimetableSchema, ITimetableSchemaWrite)
-
-    def __init__(self, day_ids, title=None, model=None):
-        """Create a new empty timetable schema.
-
-        day_ids is a sequence of the day ids of this timetable.
-
-        The caller must then assign a TimetableDay for each day ID and
-        set the model before trying to use the timetable.
-        """
-        self.title = title
-        if self.title is None:
-            self.title = "Schema"
-        self.day_ids = day_ids
-        self.days = PersistentDict()
-        self.model = model
-
-    def keys(self):
-        return list(self.day_ids)
-
-    def items(self):
-        return [(day, self.days[day]) for day in self.day_ids]
-
-    def __getitem__(self, key):
-        return self.days[key]
-
-    def __setitem__(self, key, value):
-        if not ITimetableSchemaDay.providedBy(value):
-            raise TypeError("Timetable schema can only contain"
-                            " ITimetableSchemaDay objects (got %r)" % (value,))
-        elif key not in self.day_ids:
-            raise ValueError("Key %r not in day_ids %r" % (key, self.day_ids))
-        self.days[key] = value
-
-    def createTimetable(self):
-        other = Timetable(self.day_ids)
-        other.model = self.model
-        for day_id in self.day_ids:
-            other[day_id] = TimetableDay(self[day_id].periods)
-        return other
-
-    def __eq__(self, other):
-        if ITimetableSchema.providedBy(other):
-            return (self.items() == other.items()
-                    and self.model == other.model)
-        else:
-            return False
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
-
-class TimetableSchemaDay(Persistent):
-
-    implements(ITimetableSchemaDay)
-
-    def __init__(self, periods=()):
-        self.periods = periods
-
-    def keys(self):
-        return self.periods
-
-    def items(self):
-        return [(period, Set()) for period in self.periods]
-
-    def __getitem__(self, period):
-        if period not in self.periods:
-            raise KeyError(period)
-        return Set()
-
-    def __eq__(self, other):
-        if ITimetableSchemaDay.providedBy(other):
-            return self.periods == other.periods
-        else:
-            return False
-
-    def __ne__(self, other):
-        return not self.__eq__(other)
-
 
 class Timetable(Persistent):
 
@@ -370,6 +186,19 @@ class Timetable(Persistent):
     __name__ = None
     __parent__ = None
 
+    _first = None
+    _last = None
+
+    timezone = 'UTC'
+    consecutive_periods_as_one = False
+
+    @property
+    def title(self):
+        if self.term and self.schooltt:
+            return "%s.%s" % (self.term.__name__, self.schooltt.__name__)
+        else:
+            return _("Unbound timetable.")
+
     def __init__(self, day_ids):
         """Create a new empty timetable.
 
@@ -382,6 +211,32 @@ class Timetable(Persistent):
         self.days = PersistentDict()
         self.model = None
 
+    @rwproperty.getproperty
+    def first(self):
+        if self._first is None:
+            term = getattr(self, 'term', None)
+            if term is None:
+                return None
+            return term.first
+        return self._first
+
+    @rwproperty.setproperty
+    def first(self, value):
+        self._first = value
+
+    @rwproperty.getproperty
+    def last(self):
+        if self._last is None:
+            term = getattr(self, 'term', None)
+            if term is None:
+                return None
+            return term.last
+        return self._last
+
+    @rwproperty.setproperty
+    def last(self, value):
+        self._last = value
+
     def keys(self):
         return list(self.day_ids)
 
@@ -416,31 +271,37 @@ class Timetable(Persistent):
     def update(self, other):
         if self.cloneEmpty() != other.cloneEmpty():
             raise ValueError("Timetables have different schemas")
-        for day, period, activity in other.itercontent():
-            self[day].add(period, activity, False)
+        for day, period, activity in other.activities():
+            self[day].add(period, activity, send_events=False)
+        self.first = other.first
+        self.last = other.last
 
     def cloneEmpty(self):
         other = Timetable(self.day_ids)
         other.model = self.model
+        other.timezone = self.timezone
         for day_id in self.day_ids:
-            other[day_id] = TimetableDay(self[day_id].periods)
+            other[day_id] = TimetableDay(self[day_id].periods,
+                                         self[day_id].homeroom_period_ids)
         return other
 
     def __eq__(self, other):
         if ITimetable.providedBy(other):
             return (self.items() == other.items()
-                    and self.model == other.model)
+                    and self.model == other.model
+                    and self.timezone == other.timezone)
         else:
             return False
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
-    def itercontent(self):
+    def activities(self):
+        act = []
         for day_id in self.day_ids:
             for period_id, iactivities in self.days[day_id].items():
-                for activity in iactivities:
-                    yield (day_id, period_id, activity)
+                act.extend([(day_id, period_id, activity) for activity in iactivities])
+        return act
 
 
 class TimetableDay(Persistent):
@@ -450,11 +311,17 @@ class TimetableDay(Persistent):
     timetable = None
     day_id = None
 
-    def __init__(self, periods=()):
+    def __init__(self, periods=(), homeroom_period_ids=None):
+        if homeroom_period_ids is not None:
+            for id in homeroom_period_ids:
+                assert id in periods
+        else:
+            homeroom_period_ids = []
         self.periods = periods
+        self.homeroom_period_ids = homeroom_period_ids
         self.activities = PersistentDict()
         for p in periods:
-            self.activities[p] = Set() # MaybePersistentKeysSet()
+            self.activities[p] = set() # MaybePersistentKeysSet()
 
     def keys(self):
         return self.periods
@@ -472,6 +339,9 @@ class TimetableDay(Persistent):
         activities = [act.replace(timetable=self.timetable)
                       for act in self.activities[period]]
         self.activities[period].clear()
+        # ping persistent dict, because we just modified a
+        # non-persistent set
+        self.activities[period] = self.activities[period]
         if send_events:
             for act in activities:
                 ev = TimetableActivityRemovedEvent(act, self.day_id, period)
@@ -484,9 +354,13 @@ class TimetableDay(Persistent):
         if not ITimetableActivity.providedBy(activity):
             raise TypeError("TimetableDay can only contain ITimetableActivity"
                             " objects (got %r)" % (activity, ))
+
         if activity.timetable is None:
             activity = activity.replace(timetable=self.timetable)
         self.activities[period].add(activity)
+        # ping persistent dict, because we just modified a
+        # non-persistent set
+        self.activities[period] = self.activities[period]
 
         if send_events:
             activity = activity.replace(timetable=self.timetable)
@@ -498,6 +372,9 @@ class TimetableDay(Persistent):
             raise ValueError("Key %r not in periods %r"
                              % (period, self.periods))
         self.activities[period].remove(value)
+        # ping persistent dict, because we just modified a
+        # non-persistent set
+        self.activities[period] = self.activities[period]
         if send_events:
             activity = value.replace(timetable=self.timetable)
             ev = TimetableActivityRemovedEvent(activity, self.day_id, period)
@@ -508,8 +385,10 @@ class TimetableDay(Persistent):
             return False
         if self.periods != other.periods:
             return False
+        if self.homeroom_period_ids != other.homeroom_period_ids:
+            return False
         for period in self.periods:
-            if Set(self.activities[period]) != Set(other.activities[period]):
+            if set(self.activities[period]) != set(other.activities[period]):
                 return False
         return True
 
@@ -517,7 +396,7 @@ class TimetableDay(Persistent):
         return not self.__eq__(other)
 
 
-class TimetableActivity:
+class TimetableActivity(object):
     """Timetable activity.
 
     Instances are immutable.
@@ -528,28 +407,30 @@ class TimetableActivity:
 
     implements(ITimetableActivity)
 
-    def __init__(self, title=None, owner=None, resources=(), timetable=None):
+    def __init__(self, title=None, owner=None, timetable=None, resources=None):
         self._title = title
         self._owner = owner
-        self._resources = ImmutableSet(resources)
         self._timetable = timetable
+        if resources:
+            self._resources = tuple(resources)
+        else:
+            self._resources = ()
 
     title = property(lambda self: self._title)
     owner = property(lambda self: self._owner)
-    resources = property(lambda self: self._resources)
     timetable = property(lambda self: self._timetable)
+    resources = property(lambda self: self._resources)
 
     def __repr__(self):
         return ("TimetableActivity(%r, %r, %r, %r)"
-                % (self.title, self.owner, self.resources, self.timetable))
+                % (self.title, self.owner, self.timetable, self.resources))
 
     def __eq__(self, other):
         # Is it really a good idea to ignore self.timetable?
         # On further thought it does not matter -- we never compare activities
         # that come from timetables with different keys.
         if ITimetableActivity.providedBy(other):
-            return (self.title == other.title and self.owner == other.owner
-                    and self.resources == other.resources)
+            return self.title == other.title and self.owner == other.owner
         else:
             return False
 
@@ -557,19 +438,19 @@ class TimetableActivity:
         return not self.__eq__(other)
 
     def __hash__(self):
-        return hash((self.title, self.owner, self.resources))
+        return hash((self.title, self.owner))
 
-    def replace(self, title=Unchanged, owner=Unchanged,
-                      resources=Unchanged, timetable=Unchanged):
+    def replace(self, title=Unchanged, owner=Unchanged, timetable=Unchanged,
+                resources=Unchanged):
         if title is Unchanged: title = self.title
         if owner is Unchanged: owner = self.owner
-        if resources is Unchanged: resources = self.resources
         if timetable is Unchanged: timetable = self.timetable
-        return TimetableActivity(title=title, owner=owner,
-                                 resources=resources, timetable=timetable)
+        if resources is Unchanged: resources = self.resources
+        return TimetableActivity(title=title, owner=owner, timetable=timetable,
+                                 resources=resources)
 
 
-class TimetableReplacedEvent:
+class TimetableReplacedEvent(object):
 
     implements(ITimetableReplacedEvent)
 
@@ -584,7 +465,7 @@ class TimetableReplacedEvent:
                 % (getPath(self.object), self.object.title, self.key))
 
 
-class TimetableActivityEvent:
+class TimetableActivityEvent(object):
 
     def __init__(self, activity, day_id, period_id):
         self.activity = activity
@@ -600,30 +481,32 @@ class TimetableActivityRemovedEvent(TimetableActivityEvent):
     implements(ITimetableActivityRemovedEvent)
 
 
-class SchooldayPeriod:
+class SchooldaySlot(object):
 
-    implements(ISchooldayPeriod)
+    implements(ISchooldaySlot)
 
-    def __init__(self, title, tstart, duration):
-        self.title = title
+    def __init__(self, tstart, duration):
         self.tstart = tstart
         self.duration = duration
 
     def __eq__(self, other):
-        if not ISchooldayPeriod.providedBy(other):
+        if not ISchooldaySlot.providedBy(other):
             return False
-        return (self.title == other.title and
-                self.tstart == other.tstart and
+        return (self.tstart == other.tstart and
                 self.duration == other.duration)
 
     def __ne__(self, other):
         return not (self == other)
 
+    def __cmp__(self, other):
+        return cmp((self.tstart, self.duration),
+                   (other.tstart, other.duration))
+
     def __hash__(self):
-        return hash((self.title, self.tstart, self.duration))
+        return hash((self.tstart, self.duration))
 
 
-class SchooldayTemplate:
+class SchooldayTemplate(object):
 
     # TODO: ideally, schoolday template should be an object that takes
     # a date and a period id and returns a schoolday period.  This way
@@ -633,15 +516,15 @@ class SchooldayTemplate:
     implements(ISchooldayTemplate, ISchooldayTemplateWrite)
 
     def __init__(self):
-        self.events = Set()
+        self.events = set()
 
     def __iter__(self):
-        return iter(self.events)
+        return iter(sorted(self.events))
 
     def add(self, obj):
-        if not ISchooldayPeriod.providedBy(obj):
+        if not ISchooldaySlot.providedBy(obj):
             raise TypeError("SchooldayTemplate can only contain "
-                            "ISchooldayPeriods (got %r)" % (obj,))
+                            "ISchooldaySlots (got %r)" % (obj,))
         self.events.add(obj)
 
     def remove(self, obj):
@@ -662,6 +545,32 @@ class SchooldayTemplate:
 #  Things for integrating timetabling into the core code.
 #
 
+class TimetableNameChooser(NameChooser):
+
+    implements(INameChooser)
+
+    def chooseName(self, name, obj):
+        """See INameChooser."""
+
+        i = 1
+        n = "1"
+        while n in self.context:
+            i += 1
+            n = unicode(i)
+        # Make sure the name is valid
+        self.checkName(n, obj)
+        return n
+
+
+class DuplicateTimetableError(ValueError):
+    """An error that is raised when you try to add a conflicting timetable.
+
+    Conflicting timetables are timetables that have the same school
+    timetable and term pair as any other timetable in the timetable
+    dict.
+    """
+
+
 class TimetableDict(PersistentDict):
 
     implements(ILocation, ITimetableDict)
@@ -671,10 +580,6 @@ class TimetableDict(PersistentDict):
 
     def __setitem__(self, key, value):
         assert ITimetable.providedBy(value)
-        keys = key.split(".")
-        if len(keys) != 2 or not keys[0] or not keys[1]:
-            raise ValueError("The key should be composed of a term id and a"
-                             " schema id separated with a . (got %r)" % key)
         old_value = self.get(key)
         if old_value is not None:
             old_value.__parent__ = None
@@ -710,158 +615,236 @@ class TimetableDict(PersistentDict):
     popitem = _not_implemented
 
 
-class TimetabledMixin:
-    """A mixin providing ITimetabled with the default semantics of
+TIMETABLES_KEY = 'schooltool.timetable.timetables'
+
+class TimetablesAdapter(object):
+    """This adapter adapts any annotatable object to be timetabled.
+
+    It provides ``ITimetables`` with the default semantics of
     timetable composition by membership and logic for searching for
-    ICompositeTimetableProvider facets.
+    ``ICompositeTimetableProvider`` facets.
     """
+    implements(ITimetables)
+    adapts(IOwnTimetables)
 
-    implements(ITimetabled)
+    def __init__(self, context):
+        self.object = context
+        annotations = IAnnotations(context)
+        self.timetables = annotations.get(TIMETABLES_KEY)
+        if self.timetables is None:
+            self.timetables = annotations[TIMETABLES_KEY] = TimetableDict()
+            self.timetables.__parent__ = context
 
-    def __init__(self):
-        self.timetables = TimetableDict()
-        self.timetables.__parent__ = self
+    def lookup(self, term, schooltt):
+        for timetable in self.timetables.values():
+            if (timetable.term is term and
+                timetable.schooltt is schooltt):
+                return timetable
 
-    def getCompositeTimetable(self, period_id, schema_id):
-        timetables = []
-        # Get the timetables from timetable source subscription adapters
-        for adapter in zapi.subscribers((self, ), ITimetableSource):
-            tt = adapter.getTimetable(period_id, schema_id)
-            if tt is not None:
-                timetables.append(tt)
+    @property
+    def terms(self):
+        return list(set([timetable.term
+                         for timetable in self.timetables.values()]))
 
-        # Add the individual timetable to the composite
-        try:
-            timetables.append(self.timetables["%s.%s" % (period_id, schema_id)])
-        except KeyError:
-            pass
 
-        if not timetables:
-            return None
+class CompositeTimetables(object):
+    """Adapter of IHaveTimetables to ICompositeTimetables.
 
-        # Aggregate results
-        result = timetables[0].cloneEmpty()
-        for tt in timetables:
-            result.update(tt)
+    It just wraps a IHaveTimetables object under an ICompositeTimetables
+    interface.
 
-        parent = TimetableDict()
-        parent.__parent__ = self
-        parent.__name__ = 'composite-timetables'
-        parent[".".join((period_id, schema_id))] = result
+        >>> from schooltool.timetable.tests.test_timetable import Parent
+        >>> obj = Parent()
+        >>> ct = CompositeTimetables(obj)
+        >>> ICompositeTimetables.providedBy(ct)
+        True
 
-        return result
+        >>> obj.getCompositeTimetable = lambda term, schema: [term, schema]
+        >>> ct.getCompositeTimetable("term", "schema")
+        ['term', 'schema']
 
-    def listCompositeTimetables(self):
-        keys = Set([tuple(k.split(".")) for k in self.timetables.keys()])
-        for adapter in zapi.subscribers((self, ), ITimetableSource):
-            keys |= adapter.listTimetables()
-        return keys
+    """
+    adapts(IHaveTimetables)
+    implements(ICompositeTimetables)
+
+    def __init__(self, context):
+        self.context = context
+
+    def collectTimetableSourceObjects(self):
+        objs = []
+        for adapter in subscribers((self.context, ), ITimetableSource):
+            objs.extend(adapter.getTimetableSourceObjects())
+        return set(objs)
+
+    def makeTimetableCalendar(self, first=None, last=None):
+
+        limited = (first is not None and
+                   last is not None)
+        def inRange(event):
+            return first <= event.schoolDay() <= last
 
-    def makeTimetableCalendar(self):
         events = []
-        terms = getSchoolToolApplication()["terms"]
-        for term_id, schema_id in self.listCompositeTimetables():
-            term = terms[term_id]
-            tt = self.getCompositeTimetable(term_id, schema_id)
-            cal = tt.model.createCalendar(term, tt)
-            events += list(cal)
+        for obj in self.collectTimetableSourceObjects():
+            cal = ISchoolToolCalendar(obj)
+            for event in cal:
+                if (ITimetableCalendarEvent.providedBy(event) and
+                    (not limited or inRange(event))):
+                    events.append(event)
         result = ImmutableCalendar(events)
         # Parent is needed so that we can find out the owner of this calendar.
         directlyProvides(result, ILocation)
-        result.__parent__ = self
+        result.__parent__ = self.context
         result.__name__ = 'timetable-calendar'
         return result
 
 
-class TimetableSchemaContainer(BTreeContainer):
-
-    implements(ITimetableSchemaContainer, IAttributeAnnotatable)
-
-    _default_id = None
-
-    def _set_default_id(self, new_id):
-        if new_id is not None and new_id not in self:
-            raise ValueError("Timetable schema %r does not exist" % new_id)
-        self._default_id = new_id
-
-    default_id = property(lambda self: self._default_id, _set_default_id)
-
-    def __setitem__(self, schema_id, ttschema):
-        assert ITimetableSchema.providedBy(ttschema)
-        if self.has_key(schema_id):
-            BTreeContainer.__delitem__(self, schema_id)
-        BTreeContainer.__setitem__(self, schema_id, ttschema)
-        if self.default_id is None:
-            self.default_id = schema_id
-
-    def __delitem__(self, schema_id):
-        BTreeContainer.__delitem__(self, schema_id)
-        if schema_id == self.default_id:
-            self.default_id = None
-
-    def getDefault(self):
-        return self[self.default_id]
+def getAllTimetables():
+    app = ISchoolToolApplication(None)
+    all_timetables = []
+    for ttowner in findObjectsProviding(app, IOwnTimetables):
+        timetables = queryAdapter(ttowner, ITimetables, default=None)
+        if timetables is not None:
+            all_timetables += timetables.timetables.values()
+    return all_timetables
 
 
-class TermContainer(BTreeContainer):
+def findRelatedTimetables(ob):
+    """Finds all timetables in the app instance that use a given object
 
-    implements(ITermContainer, IAttributeAnnotatable)
+    The object can be either a school timetable or a term.
 
-
-def getTermForDate(date):
-    """Find the term that contains `date`.
-
-    Returns None if `date` falls outside all terms.
+    Returns a list of Timetable objects.
     """
-    terms = getSchoolToolApplication()["terms"]
-    for term in terms.values():
-        if date in term:
-            return term
+
+    if ITerm.providedBy(ob):
+        result = []
+        for tt in getAllTimetables():
+            if sameProxiedObjects(tt.term, ob):
+                result.append(tt)
+
+    elif ITimetableSchema.providedBy(ob):
+
+        result = []
+        for tt in getAllTimetables():
+            if sameProxiedObjects(tt.schooltt, ob):
+                result.append(tt)
+
     else:
-        return None
+        raise TypeError("Expected a Term or a TimetableSchema, got %r" %
+                        (ob, ))
+
+    return result
 
 
-def getNextTermForDate(date):
-    """Find the term that contains `date`, or the next one.
+class TimetableInit(InitBase):
 
-    If there is a term that contains `date`, it is returned.  Otherwise, the
-    first term that starts after `date` is returned.  If there are none,
-    the last term that ended before `date` is returned.
-
-    Returns None if there are no terms.
-    """
-    terms = getSchoolToolApplication()["terms"]
-    before, after = [], []
-    for term in terms.values():
-        if date in term:
-            return term
-        if date > term.last:
-            before.append((term.last, term))
-        if date < term.first:
-            after.append((term.first, term))
-    if after:
-        return after[0][1]
-        return min(after)[1]
-    if before:
-        return max(before)[1]
-    return None
+    def __call__(self):
+        from schooltool.timetable.schema import TimetableSchemaContainerContainer
+        self.app['schooltool.timetable.schooltt'] = TimetableSchemaContainerContainer()
 
 
-def getPeriodsForDay(date):
-    """Return a list of timetable periods defined for `date`.
+@adapter(ITimetableDict)
+@implementer(ITerm)
+def getTermForTimetableDict(ttdict):
+    return ITerm(ttdict.__parent__)
 
-    This function uses the default timetable schema and the appropriate time
-    period for `date`.
 
-    Returns a list of ISchooldayPeriod objects.
+@adapter(ITimetable)
+@implementer(ITerm)
+def getTermForTimetable(timetable):
+    return ITerm(timetable.__parent__)
+
+
+class TimetableOverlapError(Exception):
+
+    def __init__(self, schema, first, last, overlapping):
+        self.schema = schema
+        self.first = first
+        self.last = last
+        self.overlapping = overlapping
+
+    def _formatTitle(self, timetable):
+        if (timetable.first is not None or
+            timetable.last is not None):
+            return "%s (%s-%s)" % (
+                timetable.title, timetable.first, timetable.last)
+        else:
+            return timetable.title
+
+    def __repr__(self):
+        return "Timetable %s overlaps other(s) (%s)" % (
+            '%s (%s-%s)' % (self.schema.title, self.first, self.last),
+            ", ".join(sorted(self._formatTitle(timetable)
+                             for timetable in self.overlapping)))
+
+    __str__ = __repr__
+
+
+class TimetableOverflowError(Exception):
+    def __init__(self, schema, first, last, term):
+        self.schema = schema
+        self.first = first
+        self.last = last
+        self.term = term
+
+    def __repr__(self):
+        return "Timetable %s (%s - %s) does not fit in term %s (%s - %s)" % (
+            self.schema.title, self.first, self.last,
+            self.term.title, self.term.first, self.term.last)
+
+    __str__ = __repr__
+
+
+
+class TimetableModifiedSubscriber(ObjectEventAdapterSubscriber):
+    adapts(IObjectModifiedEvent, ITimetable)
+
+    def __call__(self):
+        if ITimetableDict.providedBy(self.object.__parent__):
+            validateTimetable(self.object)
+
+
+class TimetableAddedSubscriber(ObjectEventAdapterSubscriber):
+    adapts(IObjectAddedEvent, ITimetable)
+
+    def __call__(self):
+        if ITimetableDict.providedBy(self.object.__parent__):
+            validateTimetable(self.object)
+
+
+def validateAgainstTerm(schema, first, last, term):
+    term_daterange = IDateRange(term)
+    if ((first is not None and first not in term_daterange) or
+        (last is not None and last not in term_daterange)):
+        raise TimetableOverflowError(
+            schema, first, last, term)
+
+
+def validateAgainstOthers(schema, first, last, timetables):
+    if first is None or last is None:
+        return
+
+    daterange = DateRange(first, last)
+    overlapping_timetables = []
+    for other_tt in timetables:
+        if (other_tt.schooltt is None or
+            not sameProxiedObjects(other_tt.schooltt, schema)):
+            continue
+        if (other_tt.first is None or other_tt.last is None):
+            continue
+        if daterange.overlaps(DateRange(other_tt.first, other_tt.last)):
+            overlapping_timetables.append(other_tt)
+    if overlapping_timetables:
+        raise TimetableOverlapError(
+            schema, first, last, overlapping_timetables)
+
+
+def validateTimetable(timetable):
+    validateAgainstTerm(
+        timetable.schooltt, timetable.first, timetable.last,
+        timetable.term)
+    validateAgainstOthers(
+        timetable.schooltt, timetable.first, timetable.last,
+        [tt for tt in timetable.__parent__.values()
+         if tt.__name__ != timetable.__name__])
 
-    Returns an empty list if there are no periods defined for `date` (e.g.
-    if there is no default timetable schema, or `date` falls outside all
-    time periods, or it happens to be a holiday).
-    """
-    schooldays = getTermForDate(date)
-    ttcontainer = getSchoolToolApplication()['ttschemas']
-    if ttcontainer.default_id is None or schooldays is None:
-        return []
-    ttschema = ttcontainer.getDefault()
-    return ttschema.model.periodsInDay(schooldays, ttschema, date)
