@@ -24,6 +24,9 @@ on the schooltool mailing list, in ttschema-wireframes.pdf[1].
 
     [1] http://lists.schooltool.org/pipermail/schooltool/2005-June/001347.html
 
+It has been slightly modified since.  You can find an up-to-date diagram
+of the workflow in src/schooltool/app/browser/ftests/images/
+
 The workflow is as follows:
 
     1. "New timetable schema"
@@ -43,7 +46,7 @@ The workflow is as follows:
         timetable?"
 
         Continue with step 5 if "yes".
-        Skip to step 8 if "no", and "cycle" was chosen in step 2.
+        Skip to step 6 if "no", and "cycle" was chosen in step 2.
         Skip to step 7 if "no", and "days of the week" was chosen in step 2.
 
     5. "Enter start and end times for each slot"
@@ -52,7 +55,11 @@ The workflow is as follows:
 
         Jump to step 9.
 
-    6. There is no step 6.
+    6. "Do the start and end times vary based on the day of the week, or the
+        day in the cycle?"
+
+        Continue with step 7 if "days of week".
+        Continue with step 8 if "cycle".
 
     7. "Enter the start and end times of each slot on each day:"
 
@@ -88,13 +95,21 @@ The workflow is as follows:
 
         (The user sees a grid of drop-downs.)
 
-    14. The timetable schema is created.
+    14. "Does your school have a homeroom period?"
+
+        Skip to step 16 if it doesn't.
+
+    15. "Indicate which period is the homeroom period."
+
+        (The user sees a list of drop-downs.)
+
+    16. The timetable schema is created.
 
 
-The shortest path through this workflow contains 6 steps, the longest contains
-11 steps.
+The shortest path through this workflow contains 7 steps, the longest contains
+13 steps.
 
-Step 14 needs the following data:
+Step 16 needs the following data:
 
     - title (determined in step 1)
     - timetable model factory (determined in step 2)
@@ -102,34 +117,33 @@ Step 14 needs the following data:
     - a list of periods for each day ID
     - a list of day templates (weekday -> period_id -> start time & duration)
 
-$Id: ttwizard.py 5752 2006-02-11 14:41:40Z ignas $
 """
-from sets import Set
 
 from zope.interface import Interface
 from zope.schema import TextLine, Text, getFieldNamesInOrder
 from zope.i18n import translate
-from zope.app import zapi
 from zope.app.form.utility import setUpWidgets
 from zope.app.form.utility import getWidgetsData
 from zope.app.form.interfaces import IInputWidget
 from zope.app.form.interfaces import WidgetsError
-from zope.app.publisher.browser import BrowserView
-from zope.app.pagetemplate.viewpagetemplatefile import ViewPageTemplateFile
-from zope.app.container.interfaces import INameChooser
-from zope.app.session.interfaces import ISession
+from zope.publisher.browser import BrowserView
+from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
+from zope.container.interfaces import INameChooser
+from zope.session.interfaces import ISession
+from zope.traversing.browser.absoluteurl import absoluteURL
 
-from schoolbell.app.browser.cal import day_of_week_names
+from schooltool.app.browser.cal import day_of_week_names
+from schooltool.app.interfaces import ISchoolToolApplication
+from schooltool.app.interfaces import IApplicationPreferences
+from schooltool.timetable.model import SequentialDayIdBasedTimetableModel
+from schooltool.timetable.model import SequentialDaysTimetableModel
+from schooltool.timetable.model import WeeklyTimetableModel
 from schooltool.timetable.browser import parse_time_range
 from schooltool.timetable.browser import format_time_range
-from schooltool import SchoolToolMessage as _
+from schooltool.common import SchoolToolMessage as _
 from schooltool.timetable.interfaces import ITimetableSchemaContainer
-from schooltool.timetable import TimetableSchema
-from schooltool.timetable import TimetableSchemaDay
-from schooltool.timetable import SchooldayTemplate, SchooldayPeriod
-from schooltool.timetable import WeeklyTimetableModel
-from schooltool.timetable import SequentialDaysTimetableModel
-from schooltool.timetable import SequentialDayIdBasedTimetableModel
+from schooltool.timetable.schema import TimetableSchema, TimetableSchemaDay
+from schooltool.timetable import SchooldayTemplate, SchooldaySlot
 
 
 def getSessionData(view):
@@ -261,7 +275,7 @@ class CycleStep(ChoiceStep):
         if success and session['cycle'] == 'weekly':
             weekday_names = [translate(day_of_week_names[i],
                                        context=self.request)
-                             for i in range(5)]
+                             for i in range(7)]
             session['day_names'] = weekday_names
         return success
 
@@ -284,7 +298,7 @@ class DayEntryStep(FormStep):
             self.error = _("Please enter at least one day name.")
             return False
 
-        seen = Set()
+        seen = set()
         for day in day_names:
             if day in seen:
                 self.error = _("Please make sure the day names are unique.")
@@ -317,7 +331,27 @@ class IndependentDaysStep(ChoiceStep):
             if session['cycle'] == 'weekly':
                 return WeeklySlotEntryStep(self.context, self.request)
             else:
-                return RotatingSlotEntryStep(self.context, self.request)
+                return SequentialModelStep(self.context, self.request)
+
+
+class SequentialModelStep(ChoiceStep):
+    """Step for choosing if start and end times vay based on the day of
+    the week or the day in the cycle."""
+
+    key = 'time_model'
+
+    question = _("Do start and end times vary based on the day of the week"
+                 " (Monday - Friday) or the day in the cycle?")
+
+    choices = [('weekly', _("Day of week")),
+               ('cycle_day', _("Day in cycle"))]
+
+    def next(self):
+        session = self.getSessionData()
+        if session['time_model'] == 'weekly':
+            return WeeklySlotEntryStep(self.context, self.request)
+        else:
+            return RotatingSlotEntryStep(self.context, self.request)
 
 
 def parse_name_list(names):
@@ -396,8 +430,11 @@ class SimpleSlotEntryStep(FormStep):
             self.error = _("Please enter at least one time slot.")
             return False
         session = self.getSessionData()
-        num_days = len(session['day_names'])
-        session['time_slots'] = [times] * num_days
+        if session.get('similar_days') and session['cycle'] == 'weekly':
+            session['time_slots'] = [times] * 5 + [[]] * 2
+        else:
+            num_days = len(session['day_names'])
+            session['time_slots'] = [times] * num_days
         return True
 
     def next(self):
@@ -420,6 +457,18 @@ class RotatingSlotEntryStep(Step):
     error = None
     example_intervals = '8:00 - 8:45\n9:05 - 9:50\n'
 
+    def days(self):
+        """Return a list of day ids/values for the view"""
+        for day_number in range(len(self.dayNames())):
+            day_id = "times.%s" % day_number
+            default = ""
+            if day_number < 5:
+                default = self.example_intervals
+
+            value = self.request.get(day_id, default)
+            yield {'id': day_id,
+                   'value': value}
+
     def dayNames(self):
         """Return the list of day names."""
         return self.getSessionData()['day_names']
@@ -434,10 +483,6 @@ class RotatingSlotEntryStep(Step):
                 self.error = _("Not a valid time slot: $slot.",
                                mapping={'slot': unicode(e.args[0])})
                 return False
-            if not times:
-                self.error = _("Please enter at least one time slot for $day.",
-                               mapping={'day': day_name})
-                return False
             result.append(times)
 
         session = self.getSessionData()
@@ -459,7 +504,22 @@ class WeeklySlotEntryStep(RotatingSlotEntryStep):
     def dayNames(self):
         """Return the list of day names."""
         return [translate(day_of_week_names[i], context=self.request)
-                for i in range(5)]
+                for i in range(7)]
+
+    def update(self):
+        """Store the slots in the session.
+
+        If a rotating timetable cycle is selected and slots vary based on day
+        of week, each day must have the same number of slots.
+        """
+        result = RotatingSlotEntryStep.update(self)
+        session = self.getSessionData()
+        if (result and session['cycle'] == 'rotating'
+            and session.get('time_model') == 'weekly'):
+            slots = session['time_slots']
+            assert len(slots) == 7 # Monday - Sunday
+            num_slots = len(slots[0])
+        return result
 
 
 class NamedPeriodsStep(ChoiceStep):
@@ -478,7 +538,30 @@ class NamedPeriodsStep(ChoiceStep):
         if session['named_periods']:
             return PeriodNamesStep(self.context, self.request)
         else:
-            return FinalStep(self.context, self.request)
+            periods_order = default_period_names(session['time_slots'])
+            session['periods_order'] = periods_order
+            return HomeroomStep(self.context, self.request)
+
+
+def default_period_names(time_slots):
+    """Derive period names and order from time slots.
+
+        >>> from datetime import time, timedelta
+        >>> periods = [(time(9, 0), timedelta(minutes=50)),
+        ...            (time(12, 35), timedelta(minutes=50)),
+        ...            (time(14, 15), timedelta(minutes=55))]
+        >>> periods2 = [(time(9, 10), timedelta(minutes=50)),
+        ...             (time(12, 35), timedelta(minutes=50))]
+        >>> default_period_names([periods] * 3 + [periods2])
+        [['09:00-09:50', '12:35-13:25', '14:15-15:10'],
+         ['09:00-09:50', '12:35-13:25', '14:15-15:10'],
+         ['09:00-09:50', '12:35-13:25', '14:15-15:10'],
+         ['09:10-10:00', '12:35-13:25']]
+
+    """
+    return [[format_time_range(tstart, duration)
+             for tstart, duration in slots]
+            for slots in time_slots]
 
 
 class PeriodNamesStep(FormStep):
@@ -500,7 +583,7 @@ class PeriodNamesStep(FormStep):
             self.error = _("Please enter at least $number periods.",
                            mapping={'number': min_periods})
             return False
-        seen = Set()
+        seen = set()
         for period in periods:
             if period in seen:
                 self.error = _("Please make sure the period names are unique.")
@@ -564,8 +647,8 @@ class PeriodOrderSimple(Step):
             result.append(self.request[name])
 
         # Validate that all periods are selected
-        seen = Set()
-        errors = Set()
+        seen = set()
+        errors = set()
         for period in result:
             if period not in seen:
                 seen.add(period)
@@ -582,7 +665,7 @@ class PeriodOrderSimple(Step):
         return True
 
     def next(self):
-        return FinalStep(self.context, self.request)
+        return HomeroomStep(self.context, self.request)
 
 
 class PeriodOrderComplex(Step):
@@ -619,9 +702,9 @@ class PeriodOrderComplex(Step):
             result.append(day)
 
         # Validate that all periods are selected
-        errors = Set()
+        errors = set()
         for i, day in enumerate(self.days()):
-            seen = Set()
+            seen = set()
             for period in result[i]:
                 if period not in seen:
                     seen.add(period)
@@ -638,6 +721,57 @@ class PeriodOrderComplex(Step):
         self.getSessionData()['periods_order'] = result
         return True
 
+    def next(self):
+        return HomeroomStep(self.context, self.request)
+
+
+class HomeroomStep(ChoiceStep):
+    """A step for choosing whether the school has homeroom periods."""
+
+    key = 'homeroom'
+
+    question = _("Do you check student attendance for the day in a homeroom"
+                 " period or equivalent?")
+
+    choices = ((True,  _("Yes")),
+               (False, _("No")))
+
+    def next(self):
+        session = self.getSessionData()
+        if session['homeroom']:
+            return HomeroomPeriodsStep(self.context, self.request)
+        else:
+            return FinalStep(self.context, self.request)
+
+
+class HomeroomPeriodsStep(Step):
+    """Step to indicate the homeroom period for each day."""
+
+    __call__ = ViewPageTemplateFile('templates/ttwizard_homeroom.pt')
+
+    description = _('Please indicate the homeroom period(s) for each day:')
+
+    error = None
+
+    def periodsInOrder(self):
+        return self.getSessionData()['periods_order']
+
+    def days(self):
+        session = self.getSessionData()
+        return session['day_names']
+
+    def update(self):
+        result = []
+        for daynr, periods in enumerate(self.periodsInOrder()):
+            homeroom_period_ids = []
+            for period in periods:
+                period_id = 'homeroom_%d_%s' % (daynr, period)
+                if period_id in self.request:
+                    homeroom_period_ids.append(period)
+            result.append(homeroom_period_ids)
+        self.getSessionData()['homeroom_periods'] = result
+        return True
+
     def next(self):
         return FinalStep(self.context, self.request)
 
@@ -649,7 +783,7 @@ class FinalStep(Step):
         ttschema = self.createSchema()
         self.add(ttschema)
         self.request.response.redirect(
-            zapi.absoluteURL(self.context, self.request))
+            absoluteURL(self.context, self.request))
 
     def update(self):
         return True
@@ -657,66 +791,44 @@ class FinalStep(Step):
     def next(self):
         return FirstStep(self.context, self.request)
 
-    def modelFactory(cycle, similar_days):
+    def modelFactory(cycle, similar_days, time_model):
         """Return the timetable model factory for the chosen cycle.
 
         Cycle can be either 'weekly' or 'rotating'.  Weekly cycles always
         use the WeeklyTimetableModel.
 
-            >>> FinalStep.modelFactory('weekly', None)
+            >>> FinalStep.modelFactory('weekly', None, None)
             <...WeeklyTimetableModel...>
 
-        Rotating cycles use SequentialDayIdBasedTimetableModel.
+        Rotating cycles can either vary based on the day of the week, or
+        on the day in the cycle.
 
-            >>> FinalStep.modelFactory('rotating', False)
+            >>> FinalStep.modelFactory('rotating', False, 'weekly')
+            <...SequentialDaysTimetableModel...>
+
+            >>> FinalStep.modelFactory('rotating', False, 'cycle_day')
             <...SequentialDayIdBasedTimetableModel...>
+
+        but there's no distinction between the two if all days have the same
+        time slots.
+
+            >>> FinalStep.modelFactory('rotating', True, None)
+            <...SequentialDayIdBasedTimetableModel...>
+
         """
         assert cycle in ('weekly', 'rotating')
         if cycle == 'weekly':
             return WeeklyTimetableModel
+        if similar_days:
+            return SequentialDayIdBasedTimetableModel
+        assert time_model in ('weekly', 'cycle_day')
+        if time_model == 'weekly':
+            return SequentialDaysTimetableModel
         else:
             return SequentialDayIdBasedTimetableModel
 
     modelFactory = staticmethod(modelFactory)
 
-    def periodNames(named_periods, periods_order, time_slots):
-        """Return names of periods for each day in order, in the time slot
-        cycle.
-
-        `named_periods` indicates whether periods are named (as opposed to
-        being designated by time).
-
-        `periods_order` is a list of lists of period names in order (one list
-        per day).
-
-        `time_slots` is a list of day definitions, where each day is a
-        list of slots, and each slot is a tuple (tstart, duration).
-
-            >>> FinalStep.periodNames(True, [['a', 'b', 'c']] * 3, None)
-            [['a', 'b', 'c'], ['a', 'b', 'c'], ['a', 'b', 'c']]
-
-            >>> from datetime import time, timedelta
-            >>> periods = [(time(9, 0), timedelta(minutes=50)),
-            ...            (time(12, 35), timedelta(minutes=50)),
-            ...            (time(14, 15), timedelta(minutes=55))]
-            >>> periods2 = [(time(9, 10), timedelta(minutes=50)),
-            ...             (time(12, 35), timedelta(minutes=50))]
-            >>> FinalStep.periodNames(False, None, [periods] * 3 + [periods2])
-            [['09:00-09:50', '12:35-13:25', '14:15-15:10'],
-             ['09:00-09:50', '12:35-13:25', '14:15-15:10'],
-             ['09:00-09:50', '12:35-13:25', '14:15-15:10'],
-             ['09:10-10:00', '12:35-13:25']]
-
-        """
-        if named_periods:
-            return periods_order
-        else:
-            return [[format_time_range(tstart, duration)
-                     for tstart, duration in slots]
-                    for slots in time_slots]
-
-    periodNames = staticmethod(periodNames)
-
     def dayTemplates(periods_order, time_slots):
         """Return a dict of day templates for ITimetableModelFactory.
 
@@ -728,10 +840,10 @@ class FinalStep(Step):
         """
         templates = {None: SchooldayTemplate()}
         same = True
-        for n, (periods, slots) in enumerate(zip(periods_order, time_slots)):
+        for n, slots in enumerate(time_slots):
             template = SchooldayTemplate()
-            for ptitle, (tstart, duration) in zip(periods, slots):
-                template.add(SchooldayPeriod(ptitle, tstart, duration))
+            for tstart, duration in slots:
+                template.add(SchooldaySlot(tstart, duration))
             templates[n] = template
             if n > 0 and template != templates[n-1]:
                 same = False
@@ -747,33 +859,30 @@ class FinalStep(Step):
         session = self.getSessionData()
         title = session['title']
         day_ids = session['day_names']
+        periods_order = session['periods_order']
+        if session['homeroom']:
+            homeroom_periods = session['homeroom_periods']
+        else:
+            homeroom_periods = [None] * len(day_ids)
 
-        periods_order = self.periodNames(session['named_periods'],
-                                         session.get('periods_order'),
-                                         session['time_slots'])
         day_templates = self.dayTemplates(periods_order, session['time_slots'])
 
         model_factory = self.modelFactory(session['cycle'],
-                                          session['similar_days'])
+                                          session['similar_days'],
+                                          session.get('time_model'))
         if model_factory is SequentialDayIdBasedTimetableModel:
             # This conversion is not very nice
             default = day_templates[None]
             day_templates = dict([(day_id, day_templates.get(idx, default))
                                   for idx, day_id in enumerate(day_ids)])
-        if model_factory is SequentialDaysTimetableModel:
-            # It would be better to avoid comparing model factories
-            period_names = periods_order[0]
-            for day in periods_order[1:]:
-                for name in day:
-                    if name not in period_names:
-                        period_names.append(name)
-            periods_in_days = [period_names] * len(day_ids)
-        else:
-            periods_in_days = periods_order
         model = model_factory(day_ids, day_templates)
-        ttschema = TimetableSchema(day_ids, title=title, model=model)
-        for day_id, periods in zip(day_ids, periods_in_days):
-            ttschema[day_id] = TimetableSchemaDay(periods)
+        app = ISchoolToolApplication(None)
+        tzname = IApplicationPreferences(app).timezone
+        ttschema = TimetableSchema(day_ids, title=title, model=model,
+                                   timezone=tzname)
+        for day_id, periods, hpids in zip(day_ids, periods_order,
+                                          homeroom_periods):
+            ttschema[day_id] = TimetableSchemaDay(periods, hpids)
         return ttschema
 
     def add(self, ttschema):
@@ -808,7 +917,7 @@ class TimetableSchemaWizard(BrowserView):
         if 'CANCEL' in self.request:
             self.rememberLastStep(FirstStep(self.context, self.request))
             self.request.response.redirect(
-                    zapi.absoluteURL(self.context, self.request))
+                    absoluteURL(self.context, self.request))
             return
         current_step = self.getLastStep()
         if current_step.update():
