@@ -18,24 +18,23 @@
 #
 """
 SchoolTool application interfaces
-
-$Id: interfaces.py 5752 2006-02-11 14:41:40Z ignas $
 """
 
 import datetime
 
-from zope.app.location.interfaces import ILocation
 from zope.interface import Interface, Attribute, implements
-from zope.app.container.interfaces import IContainer, IContained
-from zope.schema import Field, Object, Int, Text, TextLine, List, Set, Tuple
-from zope.schema import Dict, Date, Timedelta, Bool, Choice
+from zope.schema import Field, Object, Int, TextLine, List, Tuple
+from zope.schema import Dict, Date, Timedelta
+from zope.schema import Iterable, Bool
 from zope.schema.interfaces import IField
-from zope.app.container.constraints import contains, containers
+from zope.annotation.interfaces import IAnnotatable
+from zope.container.constraints import contains, containers
+from zope.container.interfaces import IContainer, IContained
+from zope.location.interfaces import ILocation
 
-from schoolbell.app import interfaces as sb
-from schoolbell.calendar.interfaces import Unchanged
-
-from schooltool import SchoolToolMessage as _
+from schooltool.term.interfaces import ITerm
+from schooltool.app.interfaces import ISchoolToolCalendarEvent
+from schooltool.calendar.interfaces import Unchanged
 
 #
 # Time field used in timetabling interfaces
@@ -56,92 +55,6 @@ class Time(Field):
 #  Timetabling
 #
 
-class IDateRange(Interface):
-    """A range of dates (inclusive).
-
-    If r is an IDateRange, then the following invariant holds:
-    r.first <= r.last
-
-    Note that empty date ranges cannot be represented.
-    """
-
-    first = Date(
-        title=u"The first day of the period of time covered.")
-
-    last = Date(
-        title=u"The last day of the period covered.")
-
-    def __iter__():
-        """Iterate over all dates in the range from the first to the last."""
-
-    def __contains__(date):
-        """Return True if the date is within the range, otherwise False.
-
-        Raises a TypeError if date is not a datetime.date.
-        """
-
-    def __len__():
-        """Return the number of dates covered by the range."""
-
-
-class ITerm(IDateRange, IContained):
-    """A calendar which can tell whether a day is a school day or not
-    in a certain school term.
-    """
-
-    title = TextLine(title=_("Title"))
-
-    def isSchoolday(date):
-        """Return whether the date is a schoolday.
-
-        Raises a ValueError if the date is outside of the term covered.
-        """
-
-
-class ITermWrite(Interface):
-
-    def add(day):
-        """Mark the day as a schoolday.
-
-        Raises a ValueError if the date is outside of the term covered.
-        """
-
-    def remove(day):
-        """Mark the day as a holiday.
-
-        Raises a ValueError if the date is outside of the term covered.
-        """
-
-    def reset(first, last):
-        """Change the term and mark all days as holidays.
-
-        If first is later than last, a ValueError is raised.
-        """
-
-    def addWeekdays(*weekdays):
-        """Mark that all days of week with a number in weekdays within the
-        term will be schooldays.
-
-        The numbering used is the same as one used by datetime.date.weekday()
-        method, or the calendar module: 0 is Monday, 1 is Tuesday, etc.
-        """
-
-    def removeWeekdays(*weekdays):
-        """Mark that all days of week with a number in weekdays within the
-        term will be holidays.
-
-        The numbering used is the same as one used by datetime.date.weekday()
-        method, or the calendar module: 0 is Monday, 1 is Tuesday, etc.
-        """
-
-    def toggleWeekdays(*weekdays):
-        """Toggle the state of all days of week with a number in weekdays.
-
-        The numbering used is the same as one used by datetime.date.weekday()
-        method, or the calendar module: 0 is Monday, 1 is Tuesday, etc.
-        """
-
-
 class ISchooldayTemplate(Interface):
     """A school-day template represents the times that periods are
     scheduled during a prototypical school day.
@@ -155,35 +68,26 @@ class ISchooldayTemplate(Interface):
     example, a school that has shorter school days on Wednesdays will
     have one template for Wednesdays, and one other template for
     Monday, Tuesday, Thursday and Friday.
-
-    Other schools will want to re-order the periods on different days,
-    so they will have one template with periods ABCDEF in that order,
-    and another template with periods DEFABC.
     """
 
     def __iter__():
-        """Return an iterator over the ISchooldayPeriods of this template."""
+        """Return an iterator over the ISchooldaySlots of this template."""
 
 
 class ISchooldayTemplateWrite(Interface):
     """Write access to schoolday templates."""
 
     def add(obj):
-        """Add an ISchooldayPeriod to the template.
+        """Add an ISchooldaySlot to the template.
 
-        Raises a TypeError if obj is not an ISchooldayPeriod."""
+        Raises a TypeError if obj is not an ISchooldaySlot."""
 
     def remove(obj):
         """Remove an object from the template."""
 
 
-class ISchooldayPeriod(Interface):
-    """An object binding a timetable period to a concrete time
-    interval within a schoolday template.
-    """
-
-    title = TextLine(
-        title=u"Period id of this event")
+class ISchooldaySlot(Interface):
+    """A time interval during which a period can be scheduled."""
 
     tstart = Time(
         title=u"Time of the start of the event")
@@ -192,22 +96,22 @@ class ISchooldayPeriod(Interface):
         title=u"Timedelta of the duration of the event")
 
     def __eq__(other):
-        """SchooldayPeriods are equal if all three of their
-        attributes are equal.
+        """SchooldaySlots are equal if their start times and durations
+        are equal.
 
-        Raises TypeError if other does not implement ISchooldayPeriod.
+        Raises TypeError if other does not implement ISchooldaySlot.
         """
 
     def __ne__(other):
-        """SchooldayPeriods are not equal if any of their three
-        attributes are not equal.
+        """SchooldaySlots are not equal if either their start times
+        and durations are not equal.
 
-        Raises TypeError if other does not implement ISchooldayPeriod.
+        Raises TypeError if other does not implement ISchooldaySlot.
         """
 
     def __hash__():
-        """Hashes of ISchooldayPeriods are equal iff those
-        ISchooldayPeriods are equal.
+        """Hashes of ISchooldaySlots are equal iff those
+        ISchooldaySlots are equal.
         """
 
 
@@ -240,13 +144,11 @@ class ITimetableModel(Interface):
     exceptionDays = Dict(
         title=u"Exception schoolday templates",
         key_type=Date(title=u"Date"),
-        value_type=Object(title=u"Schoolday template",
-                          schema=ISchooldayTemplate),
+        value_type=List(title=u"Schoolday template"),
         description=u"""
-        Schoolday templates for special days.
+        Timetables for special days
 
-        Schoolday templates provided in this attribute override the
-        templates in the dayTemplates attribute.
+        The values are lists of tuples of (period_id, SchooldaySlot).
         """)
 
     exceptionDayIds = Dict(
@@ -262,12 +164,15 @@ class ITimetableModel(Interface):
         is a replacement for some other day.
         """)
 
-    def createCalendar(term, timetable):
+    def createCalendar(term, timetable, first=None, last=None):
         """Return an ICalendar composed out of term and timetable.
 
         This method has model-specific knowledge as to how the schooldays,
         weekends and holidays map affects the mapping of the timetable
         onto the real-world calendar.
+
+        You can specify a range of dates to generate a calendar spanning a
+        given date range (inclusive) only.
         """
 
     def periodsInDay(term, timetable, date):
@@ -344,9 +249,8 @@ class ITimetableActivity(Interface):
         The activity lives in the owner's timetable.
         """)
 
-    resources = Set(
+    resources = Iterable(
         title=u"A set of resources assigned to this activity.",
-        value_type=Field(title=u"A resource"),
         description=u"""
         The activity is also present in the timetables of all resources
         assigned to this activity.
@@ -359,14 +263,13 @@ class ITimetableActivity(Interface):
         to a composite timetable or a timetable of a resource.
         """)
 
-    def replace(title=Unchanged, owner=Unchanged, resources=Unchanged,
-                timetable=Unchanged):
+    def replace(title=Unchanged, owner=Unchanged, timetable=Unchanged):
         """Return a copy of this activity with some fields changed."""
 
     def __eq__(other):
         """Is this timetable activity equal to `other`?
 
-        Timetable activities are equal iff their title, owner and resources
+        Timetable activities are equal iff their title and owner
         attributes are equal.
 
         Returns false if other is not a timetable activity.
@@ -430,6 +333,8 @@ class ITimetableSchema(IContained):
         title=u"A timetable model this timetable should be used with.",
         schema=ITimetableModel)
 
+    timezone = TextLine(title=u"The name of a timezone of this timetable")
+
     def keys():
         """Return a sequence of identifiers for days within the timetable.
 
@@ -445,11 +350,13 @@ class ITimetableSchema(IContained):
     def __getitem__(key):
         """Return a ITimetableSchemaDay for a given day id."""
 
-    def createTimetable():
+    def createTimetable(term):
         """Return a new empty timetable with the same structure.
 
         The new timetable has the same set of day_ids, and the sets of
         period ids within each day.  It has no activities.
+
+        The new timetable is bound to the term passed as the argument.
         """
 
 
@@ -480,6 +387,10 @@ class ITimetableSchemaDay(Interface):
         title=u"A list of period IDs for this day.",
         value_type=TextLine(title=u"A period id"))
 
+    homeroom_period_ids = List(
+        title=u"A list of homeroom period IDs for this day",
+        required=False)
+
     def keys():
         """Return self.periods."""
 
@@ -500,11 +411,34 @@ class ITimetable(ILocation):
     A timetable represents the repeating lesson schedule for just one
     pupil, or one teacher, or one bookable resource.
     """
+    containers('.ITimetableDict')
+
+    title = TextLine(title=u"Title", required=True)
+
+    first = Date(
+        title=u"Apply timetable from")
+
+    last = Date(
+        title=u"Apply timetable until")
 
     model = Object(
         title=u"A timetable model this timetable should be used with.",
         schema=ITimetableModel)
 
+    timezone = TextLine(title=u"The name of a timezone of this timetable")
+
+    consecutive_periods_as_one = Bool(
+        title=u"Treat consecutive periods as a single meeting",
+        required=False)
+
+    term = Object(
+        title=u"The term this timetable is for.",
+        schema=ITerm)
+
+    schooltt = Object(
+        title=u"The school timetable this timetable is for.",
+        schema=ITimetableSchema)
+
     def keys():
         """Return a sequence of identifiers for days within the timetable.
 
@@ -520,10 +454,10 @@ class ITimetable(ILocation):
     def __getitem__(key):
         """Return a ITimetableDay for a given day id."""
 
-    def itercontent():
-        """Iterate over all activites in this timetable.
+    def activities():
+        """Return all activities in this timetable.
 
-        Return an iterator for tuples (day_id, period_id, activity).
+        Returns a list of tuples (day_id, period_id, activity).
         """
 
     def cloneEmpty():
@@ -595,6 +529,10 @@ class ITimetableDay(Interface):
         title=u"A list of period IDs for this day.",
         value_type=TextLine(title=u"A period id"))
 
+    homeroom_period_ids = TextLine(
+        title=u"IDs of homeroom periods",
+        required=False)
+
     def keys():
         """Return self.periods."""
 
@@ -671,9 +609,12 @@ class ITimetableActivityRemovedEvent(ITimetableActivityEvent):
     """Event that gets sent when an activity is removed from a timetable."""
 
 
-class ITimetableCalendarEvent(sb.ISchoolBellCalendarEvent):
+class ITimetableCalendarEvent(ISchoolToolCalendarEvent):
     """A calendar event that has been created from a timetable."""
 
+    day_id = TextLine(
+        title=u"The id of the timetable day.")
+
     period_id = TextLine(
         title=u"The period id of the corresponding timetable event.")
 
@@ -682,7 +623,7 @@ class ITimetableCalendarEvent(sb.ISchoolBellCalendarEvent):
         schema=ITimetableActivity)
 
 
-class ITimetableDict(IContainer):
+class ITimetableDict(IContainer, ILocation):
     """Container for timetables.
 
     The id of the timetable is composed by joining term id and
@@ -690,13 +631,36 @@ class ITimetableDict(IContainer):
     means a timetable for a term "2005-fall" with a timetable schema
     "default".
     """
+    contains(ITimetable)
 
 
-class ITimetabled(Interface):
-    """An object that has a timetable related to it -- either its own,
+class IHaveTimetables(Interface):
+    """A marker interface for content objects to declare themselves as having
+       timetables."""
+
+
+class IOwnTimetables(IHaveTimetables, IAnnotatable):
+    """A marker interface for content objects to declare themselves as having
+       timetables."""
+
+
+class IBookResources(Interface):
+    """An object that can have booked resources."""
+
+    resources = Iterable(
+        title=u"Booked resources")
+
+
+class ITimetables(Interface):
+    """The timetable content related to an object -- either its own,
     or composed of the timetables of related objects.
     """
 
+    object = Attribute("The object these timetables are for.")
+
+    terms = Attribute(
+        """A list of terms this object was scheduled for.""")
+
     timetables = Object(
         schema=ITimetableDict,
         title=u"Private timetables of this object",
@@ -711,24 +675,22 @@ class ITimetabled(Interface):
         getCompositeTimetable).
         """)
 
-    def getCompositeTimetable(term_id, tt_schema_id):
-        """Return a composite timetable for a given object with a
-        given timetable schema for a given time period id.
+    def lookup(term, schooltt):
+        """ """
 
-        The timetable returned includes the events from the timetables
-        of parent groups, groups taught, etc.
+class ICompositeTimetables(Interface):
+    """An interface for objects that have composite timetables."""
 
-        This function can return None if the object has no timetable.
+    def collectTimetableSourceObjects():
+        """Collect objects timetables of which are included in the composite timetable."""
+
+    def makeTimetableCalendar(first=None, last=None):
+        """Generate and return a calendar from all composite timetables.
+
+        You can specify a range of dates to generate a calendar spanning a
+        given date range (inclusive) only.
         """
 
-    def listCompositeTimetables():
-        """Return a sequence of (time_period_id, tt_schema_id) for all
-        available composite timetables.
-        """
-
-    def makeTimetableCalendar():
-        """Generate and return a calendar from all composite timetables."""
-
 
 class ITimetableSource(Interface):
     """Timetable source.
@@ -737,18 +699,9 @@ class ITimetableSource(Interface):
     composite timetable of the context.
     """
 
-    def getTimetable(term_id, schema_id):
-        """Return a timetable to be merged to the composite timetable
-        of the context.
-
-        Can return None if this adapter can not contribute to the
-        timetable of this object.
-        """
-
-    def listTimetables():
-        """Return a sequence of timetable keys for which getTimetable
-        would return not a None.
-        """
+    def getTimetableSourceObjects():
+        """Return objects calendars of which will be merged to the composite
+        timetable of the context."""
 
 
 class ITimetableReplacedEvent(Interface):
@@ -756,7 +709,7 @@ class ITimetableReplacedEvent(Interface):
 
     object = Object(
         title=u"The owner of the timetable.",
-        schema=ITimetabled)
+        schema=ITimetables)
 
     key = Tuple(
         title=u"Tuple (time_period_id, schema_id).",
@@ -804,11 +757,11 @@ class ITimetableSchemaContainer(IContainer, ILocation):
         """Return the default schema for the school"""
 
 
-class ITermContainer(IContainer, ILocation):
-    """A container for terms.
+class ITimetableSchemaContainerContainer(IContainer, ILocation):
+    contains(ITimetableSchemaContainer)
 
-    It stores term calendars for registered term IDs.
-    """
 
-    contains(ITerm)
+class ITimetableSchemaContained(ITimetableSchema, IContained):
+    """Timetable Schema contained in an ITimetableSchemaContainer."""
 
+    containers(ITimetableSchemaContainer)
